#
# findex.rb
#
#   Copyright (c) 2001 Minero Aoki <aamine@loveruby.net>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'refe/completion'


module ReFe

  class FunctionIndex

    def initialize
      @comptable = FCompletionTable.new
      @descriptions = nil
      @definitions = nil
    end

    attr_accessor :descriptions
    attr_accessor :definitions

    def inspect
      "#<#{type}>"
    end

    #
    # registration
    #

    def register_description( name, desc )
      @descriptions[ name ] = desc
      @comptable.add name
    end

    def register_definition( name, header, body, meta = nil )
      @definitions[ name + '-header' ] = header
      @definitions[ name + '-body' ] = body
      @definitions[ name + '-meta' ] = meta if meta
      @comptable.add name
    end

    def desc_updating( &block )
      @desciptions.updating( &block )
    end

    def def_updating( &block )
      @definitions.updating( &block )
    end

    #
    # completion
    #

    def complete( name )
      k = (/\A[\-_]/ === name) ? 'rb_' + name[1..-1] : name
      cand = @comptable.complete(name)
      cand.empty? and notfound name
      cand
    end

    #
    # lookup
    #

    def description( name )
      @descriptions[name] or notfound name
    end

    def header( name )
      @definitions[ name + '-header' ] or notfound name
    end
    
    def definition( name )
      @definitions[ name + '-body' ]   or notfound name
    end

    def meta( name )
      @definitions[ name + '-meta' ]
    end

    def notfound( name )
      raise SearchError, "function not registered: #{name}"
    end
  
  end   # class FuntionIndex

end
